@@ -1,28 +1,30 @@
-use std::borrow::Cow;
-use std::convert::{TryFrom, TryInto};
-use std::ffi::{CStr, OsStr};
-use std::fs::File;
-use std::io::BufReader;
+use std::ffi::CStr;
+use std::fmt::{Debug, Formatter};
 use std::slice;
 use std::sync::Arc;
-use std::time::SystemTime;
 
 use libc::{c_char, size_t};
-use rustls::client::{ResolvesClientCert, ServerCertVerified, ServerCertVerifier};
+use pki_types::{CertificateDer, UnixTime};
+use rustls::client::danger::{HandshakeSignatureValid, ServerCertVerified, ServerCertVerifier};
+use rustls::client::ResolvesClientCert;
+use rustls::crypto::ring::ALL_CIPHER_SUITES;
 use rustls::{
-    sign::CertifiedKey, Certificate, CertificateError, ClientConfig, ClientConnection,
-    ProtocolVersion, RootCertStore, SupportedCipherSuite, WantsVerifier, ALL_CIPHER_SUITES,
+    sign::CertifiedKey, CertificateError, ClientConfig, ClientConnection, DigitallySignedStruct,
+    Error, ProtocolVersion, SignatureScheme, SupportedCipherSuite, WantsVerifier,
 };
 
-use crate::cipher::{rustls_certified_key, rustls_root_cert_store, rustls_supported_ciphersuite};
+use crate::cipher::{
+    rustls_certified_key, rustls_server_cert_verifier, rustls_supported_ciphersuite,
+};
 use crate::connection::{rustls_connection, Connection};
 use crate::error::rustls_result::{InvalidParameter, NullParameter};
 use crate::error::{self, rustls_result};
 use crate::rslice::NulByte;
 use crate::rslice::{rustls_slice_bytes, rustls_slice_slice_bytes, rustls_str};
 use crate::{
-    ffi_panic_boundary, try_arc_from_ptr, try_box_from_ptr, try_mut_from_ptr, try_ref_from_ptr,
-    try_slice, userdata_get, ArcCastPtr, BoxCastPtr, CastConstPtr, CastPtr,
+    ffi_panic_boundary, free_arc, free_box, set_boxed_mut_ptr, to_arc_const_ptr, to_boxed_mut_ptr,
+    try_box_from_ptr, try_clone_arc, try_mut_from_ptr, try_ref_from_ptr, try_slice, userdata_get,
+    Castable, OwnershipArc, OwnershipBox,
 };
 
 /// A client config being constructed. A builder can be modified by,
@@ -33,9 +35,6 @@ use crate::{
 /// `Box<ClientConfig>`.
 /// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
 pub struct rustls_client_config_builder {
-    // We use the opaque struct pattern to tell C about our types without
-    // telling them what's inside.
-    // https://doc.rust-lang.org/nomicon/ffi.html#representing-opaque-structs
     _private: [u8; 0],
 }
 
@@ -47,44 +46,63 @@ pub(crate) struct ClientConfigBuilder {
     cert_resolver: Option<Arc<dyn rustls::client::ResolvesClientCert>>,
 }
 
-impl CastPtr for rustls_client_config_builder {
+impl Castable for rustls_client_config_builder {
+    type Ownership = OwnershipBox;
     type RustType = ClientConfigBuilder;
 }
 
-impl BoxCastPtr for rustls_client_config_builder {}
-
 /// A client config that is done being constructed and is now read-only.
 /// Under the hood, this object corresponds to an `Arc<ClientConfig>`.
 /// <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html>
 pub struct rustls_client_config {
-    // We use the opaque struct pattern to tell C about our types without
-    // telling them what's inside.
-    // https://doc.rust-lang.org/nomicon/ffi.html#representing-opaque-structs
     _private: [u8; 0],
 }
 
-impl CastConstPtr for rustls_client_config {
+impl Castable for rustls_client_config {
+    type Ownership = OwnershipArc;
     type RustType = ClientConfig;
 }
 
-impl ArcCastPtr for rustls_client_config {}
-
+#[derive(Debug)]
 struct NoneVerifier;
 
 impl ServerCertVerifier for NoneVerifier {
     fn verify_server_cert(
         &self,
-        _end_entity: &Certificate,
-        _intermediates: &[Certificate],
-        _server_name: &rustls::ServerName,
-        _scts: &mut dyn Iterator<Item = &[u8]>,
+        _end_entity: &CertificateDer,
+        _intermediates: &[CertificateDer],
+        _server_name: &pki_types::ServerName<'_>,
         _ocsp_response: &[u8],
-        _now: SystemTime,
+        _now: UnixTime,
     ) -> Result<ServerCertVerified, rustls::Error> {
         Err(rustls::Error::InvalidCertificate(
             CertificateError::BadSignature,
         ))
     }
+
+    fn verify_tls12_signature(
+        &self,
+        _message: &[u8],
+        _cert: &CertificateDer,
+        _dss: &DigitallySignedStruct,
+    ) -> Result<HandshakeSignatureValid, Error> {
+        Err(Error::InvalidCertificate(CertificateError::BadSignature))
+    }
+
+    fn verify_tls13_signature(
+        &self,
+        _message: &[u8],
+        _cert: &CertificateDer,
+        _dss: &DigitallySignedStruct,
+    ) -> Result<HandshakeSignatureValid, Error> {
+        Err(Error::InvalidCertificate(CertificateError::BadSignature))
+    }
+
+    fn supported_verify_schemes(&self) -> Vec<SignatureScheme> {
+        rustls::crypto::ring::default_provider()
+            .signature_verification_algorithms
+            .supported_schemes()
+    }
 }
 
 impl rustls_client_config_builder {
@@ -100,13 +118,13 @@ impl rustls_client_config_builder {
     pub extern "C" fn rustls_client_config_builder_new() -> *mut rustls_client_config_builder {
         ffi_panic_boundary! {
             let builder = ClientConfigBuilder {
-                base: rustls::ClientConfig::builder().with_safe_defaults(),
+                base: rustls::ClientConfig::builder(),
                 verifier: Arc::new(NoneVerifier),
                 cert_resolver: None,
                 alpn_protocols: vec![],
                 enable_sni: true,
             };
-            BoxCastPtr::to_mut_ptr(builder)
+            to_boxed_mut_ptr(builder)
         }
     }
 
@@ -135,7 +153,8 @@ impl rustls_client_config_builder {
         builder_out: *mut *mut rustls_client_config_builder,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let cipher_suites: &[*const rustls_supported_ciphersuite] = try_slice!(cipher_suites, cipher_suites_len);
+            let cipher_suites: &[*const rustls_supported_ciphersuite] =
+                try_slice!(cipher_suites, cipher_suites_len);
             let mut cs_vec: Vec<SupportedCipherSuite> = Vec::new();
             for &cs in cipher_suites.iter() {
                 let cs = try_ref_from_ptr!(cs);
@@ -156,7 +175,12 @@ impl rustls_client_config_builder {
                 }
             }
 
-            let result = rustls::ClientConfig::builder().with_cipher_suites(&cs_vec).with_safe_default_kx_groups().with_protocol_versions(&versions);
+            let provider = rustls::crypto::CryptoProvider {
+                cipher_suites: cs_vec,
+                ..rustls::crypto::ring::default_provider()
+            };
+            let result = rustls::ClientConfig::builder_with_provider(provider.into())
+                .with_protocol_versions(&versions);
             let base = match result {
                 Ok(new) => new,
                 Err(_) => return rustls_result::InvalidParameter,
@@ -169,7 +193,7 @@ impl rustls_client_config_builder {
                 enable_sni: true,
             };
 
-            BoxCastPtr::set_mut_ptr(builder_out, config_builder);
+            set_boxed_mut_ptr(builder_out, config_builder);
             rustls_result::Ok
         }
     }
@@ -226,22 +250,17 @@ unsafe impl Send for Verifier {}
 /// rustls_client_config_builder_dangerous_set_certificate_verifier.
 unsafe impl Sync for Verifier {}
 
-impl rustls::client::ServerCertVerifier for Verifier {
+impl ServerCertVerifier for Verifier {
     fn verify_server_cert(
         &self,
-        end_entity: &Certificate,
-        intermediates: &[Certificate],
-        server_name: &rustls::ServerName,
-        _scts: &mut dyn Iterator<Item = &[u8]>,
+        end_entity: &CertificateDer,
+        intermediates: &[CertificateDer],
+        server_name: &pki_types::ServerName<'_>,
         ocsp_response: &[u8],
-        _now: SystemTime,
+        _now: UnixTime,
     ) -> Result<ServerCertVerified, rustls::Error> {
         let cb = self.callback;
-        let server_name: Cow<'_, str> = match server_name {
-            rustls::ServerName::DnsName(n) => n.as_ref().into(),
-            rustls::ServerName::IpAddress(ip) => ip.to_string().into(),
-            _ => return Err(rustls::Error::General("unknown name type".to_string())),
-        };
+        let server_name = server_name.to_str();
         let server_name: rustls_str = match server_name.as_ref().try_into() {
             Ok(r) => r,
             Err(NulByte {}) => return Err(rustls::Error::General("NUL byte in SNI".to_string())),
@@ -263,13 +282,51 @@ impl rustls::client::ServerCertVerifier for Verifier {
             rustls::Error::General("internal error with thread-local storage".to_string())
         })?;
         let result: u32 = unsafe { cb(userdata, &params) };
-        let result: rustls_result =
-            rustls_result::try_from(result).unwrap_or(rustls_result::General);
-        match result {
+        match rustls_result::from(result) {
             rustls_result::Ok => Ok(ServerCertVerified::assertion()),
             r => Err(error::cert_result_to_error(r)),
         }
     }
+
+    fn verify_tls12_signature(
+        &self,
+        message: &[u8],
+        cert: &CertificateDer,
+        dss: &DigitallySignedStruct,
+    ) -> Result<HandshakeSignatureValid, Error> {
+        rustls::crypto::verify_tls12_signature(
+            message,
+            cert,
+            dss,
+            &rustls::crypto::ring::default_provider().signature_verification_algorithms,
+        )
+    }
+
+    fn verify_tls13_signature(
+        &self,
+        message: &[u8],
+        cert: &CertificateDer,
+        dss: &DigitallySignedStruct,
+    ) -> Result<HandshakeSignatureValid, Error> {
+        rustls::crypto::verify_tls13_signature(
+            message,
+            cert,
+            dss,
+            &rustls::crypto::ring::default_provider().signature_verification_algorithms,
+        )
+    }
+
+    fn supported_verify_schemes(&self) -> Vec<SignatureScheme> {
+        rustls::crypto::ring::default_provider()
+            .signature_verification_algorithms
+            .supported_schemes()
+    }
+}
+
+impl Debug for Verifier {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Verifier").finish()
+    }
 }
 
 impl rustls_client_config_builder {
@@ -309,72 +366,24 @@ impl rustls_client_config_builder {
                 None => return rustls_result::InvalidParameter,
             };
 
-            let verifier: Verifier = Verifier{callback};
+            let verifier: Verifier = Verifier { callback };
             config_builder.verifier = Arc::new(verifier);
             rustls_result::Ok
         }
     }
 
-    /// Use the trusted root certificates from the provided store.
+    /// Configure the server certificate verifier.
     ///
-    /// This replaces any trusted roots already configured with copies
-    /// from `roots`. This adds 1 to the refcount for `roots`. When you
-    /// call rustls_client_config_free or rustls_client_config_builder_free,
-    /// those will subtract 1 from the refcount for `roots`.
+    /// This increases the reference count of `verifier` and does not take ownership.
     #[no_mangle]
-    pub extern "C" fn rustls_client_config_builder_use_roots(
-        config_builder: *mut rustls_client_config_builder,
-        roots: *const rustls_root_cert_store,
-    ) -> rustls_result {
+    pub extern "C" fn rustls_client_config_builder_set_server_verifier(
+        builder: *mut rustls_client_config_builder,
+        verifier: *const rustls_server_cert_verifier,
+    ) {
         ffi_panic_boundary! {
-            let builder = try_mut_from_ptr!(config_builder);
-            let root_store: &RootCertStore = try_ref_from_ptr!(roots);
-            builder.verifier = Arc::new(rustls::client::WebPkiVerifier::new(root_store.clone(), None));
-            rustls_result::Ok
-        }
-    }
-
-    /// Add trusted root certificates from the named file, which should contain
-    /// PEM-formatted certificates.
-    #[no_mangle]
-    pub extern "C" fn rustls_client_config_builder_load_roots_from_file(
-        config_builder: *mut rustls_client_config_builder,
-        filename: *const c_char,
-    ) -> rustls_result {
-        ffi_panic_boundary! {
-            let config_builder = try_mut_from_ptr!(config_builder);
-            let filename: &CStr = unsafe {
-                if filename.is_null() {
-                    return rustls_result::NullParameter;
-                }
-                CStr::from_ptr(filename)
-            };
-
-            let filename: &[u8] = filename.to_bytes();
-            let filename: &str = match std::str::from_utf8(filename) {
-                Ok(s) => s,
-                Err(_) => return rustls_result::Io,
-            };
-            let filename: &OsStr = OsStr::new(filename);
-            let mut cafile = match File::open(filename) {
-                Ok(f) => f,
-                Err(_) => return rustls_result::Io,
-            };
-
-            let mut bufreader = BufReader::new(&mut cafile);
-            let certs = match rustls_pemfile::certs(&mut bufreader) {
-                Ok(certs) => certs,
-                Err(_) => return rustls_result::Io,
-            };
-
-            let mut roots = RootCertStore::empty();
-            let (_, failed) = roots.add_parsable_certificates(&certs);
-            if failed > 0 {
-                return rustls_result::CertificateParseError;
-            }
-
-            config_builder.verifier = Arc::new(rustls::client::WebPkiVerifier::new(roots, None));
-            rustls_result::Ok
+            let builder: &mut ClientConfigBuilder = try_mut_from_ptr!(builder);
+            let verifier = try_ref_from_ptr!(verifier);
+            builder.verifier = verifier.clone();
         }
     }
 
@@ -442,10 +451,11 @@ impl rustls_client_config_builder {
     ) -> rustls_result {
         ffi_panic_boundary! {
             let config: &mut ClientConfigBuilder = try_mut_from_ptr!(builder);
-            let keys_ptrs: &[*const rustls_certified_key] = try_slice!(certified_keys, certified_keys_len);
+            let keys_ptrs: &[*const rustls_certified_key] =
+                try_slice!(certified_keys, certified_keys_len);
             let mut keys: Vec<Arc<CertifiedKey>> = Vec::new();
             for &key_ptr in keys_ptrs {
-                let certified_key: Arc<CertifiedKey> = try_arc_from_ptr!(key_ptr);
+                let certified_key: Arc<CertifiedKey> = try_clone_arc!(key_ptr);
                 keys.push(certified_key);
             }
             config.cert_resolver = Some(Arc::new(ResolvesClientCertFromChoices { keys }));
@@ -455,6 +465,7 @@ impl rustls_client_config_builder {
 }
 
 /// Always send the same client certificate.
+#[derive(Debug)]
 struct ResolvesClientCertFromChoices {
     keys: Vec<Arc<CertifiedKey>>,
 }
@@ -487,14 +498,17 @@ impl rustls_client_config_builder {
     ) -> *const rustls_client_config {
         ffi_panic_boundary! {
             let builder: Box<ClientConfigBuilder> = try_box_from_ptr!(builder);
-            let config = builder.base.with_custom_certificate_verifier(builder.verifier);
+            let config = builder
+                .base
+                .dangerous()
+                .with_custom_certificate_verifier(builder.verifier);
             let mut config = match builder.cert_resolver {
                 Some(r) => config.with_client_cert_resolver(r),
                 None => config.with_no_client_auth(),
             };
             config.alpn_protocols = builder.alpn_protocols;
             config.enable_sni = builder.enable_sni;
-            ArcCastPtr::to_const_ptr(config)
+            to_arc_const_ptr(config)
         }
     }
 
@@ -506,7 +520,7 @@ impl rustls_client_config_builder {
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_free(config: *mut rustls_client_config_builder) {
         ffi_panic_boundary! {
-            BoxCastPtr::to_box(config);
+            free_box(config);
         }
     }
 }
@@ -521,7 +535,7 @@ impl rustls_client_config {
     #[no_mangle]
     pub extern "C" fn rustls_client_config_free(config: *const rustls_client_config) {
         ffi_panic_boundary! {
-            rustls_client_config::free(config);
+            free_arc(config);
         }
     }
 
@@ -542,29 +556,29 @@ impl rustls_client_config {
         conn_out: *mut *mut rustls_connection,
     ) -> rustls_result {
         ffi_panic_boundary! {
-        let server_name: &CStr = unsafe {
-            if server_name.is_null() {
-                return NullParameter;
-            }
-            CStr::from_ptr(server_name)
-        };
-        let config: Arc<ClientConfig> = try_arc_from_ptr!(config);
-        let server_name: &str = match server_name.to_str() {
-            Ok(s) => s,
-            Err(std::str::Utf8Error { .. }) => return rustls_result::InvalidDnsNameError,
-        };
-        let server_name: rustls::ServerName = match server_name.try_into() {
-            Ok(sn) => sn,
-            Err(_) => return rustls_result::InvalidDnsNameError,
-        };
-        let client = ClientConnection::new(config, server_name).unwrap();
+            let server_name: &CStr = unsafe {
+                if server_name.is_null() {
+                    return NullParameter;
+                }
+                CStr::from_ptr(server_name)
+            };
+            let config: Arc<ClientConfig> = try_clone_arc!(config);
+            let server_name: &str = match server_name.to_str() {
+                Ok(s) => s,
+                Err(std::str::Utf8Error { .. }) => return rustls_result::InvalidDnsNameError,
+            };
+            let server_name: pki_types::ServerName = match server_name.try_into() {
+                Ok(sn) => sn,
+                Err(_) => return rustls_result::InvalidDnsNameError,
+            };
+            let client = ClientConnection::new(config, server_name).unwrap();
 
-        // We've succeeded. Put the client on the heap, and transfer ownership
-        // to the caller. After this point, we must return rustls_result::Ok so the
-        // caller knows it is responsible for this memory.
-        let c = Connection::from_client(client);
-        BoxCastPtr::set_mut_ptr(conn_out, c);
-        rustls_result::Ok
+            // We've succeeded. Put the client on the heap, and transfer ownership
+            // to the caller. After this point, we must return rustls_result::Ok so the
+            // caller knows it is responsible for this memory.
+            let c = Connection::from_client(client);
+            set_boxed_mut_ptr(conn_out, c);
+            rustls_result::Ok
         }
     }
 }
