@@ -1,23 +1,21 @@
-use std::convert::TryInto;
 use std::ffi::c_void;
+use std::fmt::{Debug, Formatter};
 use std::ptr::null;
 use std::slice;
 use std::sync::Arc;
 
 use libc::size_t;
+use rustls::crypto::ring::ALL_CIPHER_SUITES;
+use rustls::server::danger::ClientCertVerifier;
 use rustls::server::{
-    AllowAnyAnonymousOrAuthenticatedClient, AllowAnyAuthenticatedClient, ClientCertVerifier,
-    ClientHello, NoClientAuth, ResolvesServerCert, ServerConfig, ServerConnection,
-    StoresServerSessions,
+    ClientHello, ResolvesServerCert, ServerConfig, ServerConnection, StoresServerSessions,
+    WebPkiClientVerifier,
 };
 use rustls::sign::CertifiedKey;
-use rustls::{
-    ProtocolVersion, SignatureScheme, SupportedCipherSuite, WantsVerifier, ALL_CIPHER_SUITES,
-};
+use rustls::{ProtocolVersion, SignatureScheme, SupportedCipherSuite, WantsVerifier};
 
 use crate::cipher::{
-    rustls_certified_key, rustls_client_cert_verifier, rustls_client_cert_verifier_optional,
-    rustls_supported_ciphersuite,
+    rustls_certified_key, rustls_client_cert_verifier, rustls_supported_ciphersuite,
 };
 use crate::connection::{rustls_connection, Connection};
 use crate::error::rustls_result::{InvalidParameter, NullParameter};
@@ -28,8 +26,9 @@ use crate::session::{
     SessionStoreGetCallback, SessionStorePutCallback,
 };
 use crate::{
-    ffi_panic_boundary, try_arc_from_ptr, try_box_from_ptr, try_mut_from_ptr, try_ref_from_ptr,
-    try_slice, userdata_get, ArcCastPtr, BoxCastPtr, CastConstPtr, CastPtr,
+    ffi_panic_boundary, free_arc, free_box, set_boxed_mut_ptr, to_arc_const_ptr, to_boxed_mut_ptr,
+    try_box_from_ptr, try_clone_arc, try_mut_from_ptr, try_ref_from_ptr, try_slice, userdata_get,
+    Castable, OwnershipArc, OwnershipBox, OwnershipRef,
 };
 
 /// A server config being constructed. A builder can be modified by,
@@ -39,9 +38,6 @@ use crate::{
 /// for concurrent mutation.
 /// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
 pub struct rustls_server_config_builder {
-    // We use the opaque struct pattern to tell C about our types without
-    // telling them what's inside.
-    // https://doc.rust-lang.org/nomicon/ffi.html#representing-opaque-structs
     _private: [u8; 0],
 }
 
@@ -54,28 +50,23 @@ pub(crate) struct ServerConfigBuilder {
     ignore_client_order: Option<bool>,
 }
 
-impl CastPtr for rustls_server_config_builder {
+impl Castable for rustls_server_config_builder {
+    type Ownership = OwnershipBox;
     type RustType = ServerConfigBuilder;
 }
 
-impl BoxCastPtr for rustls_server_config_builder {}
-
 /// A server config that is done being constructed and is now read-only.
 /// Under the hood, this object corresponds to an `Arc<ServerConfig>`.
 /// <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html>
 pub struct rustls_server_config {
-    // We use the opaque struct pattern to tell C about our types without
-    // telling them what's inside.
-    // https://doc.rust-lang.org/nomicon/ffi.html#representing-opaque-structs
     _private: [u8; 0],
 }
 
-impl CastConstPtr for rustls_server_config {
+impl Castable for rustls_server_config {
+    type Ownership = OwnershipArc;
     type RustType = ServerConfig;
 }
 
-impl ArcCastPtr for rustls_server_config {}
-
 impl rustls_server_config_builder {
     /// Create a rustls_server_config_builder. Caller owns the memory and must
     /// eventually call rustls_server_config_builder_build, then free the
@@ -85,14 +76,14 @@ impl rustls_server_config_builder {
     pub extern "C" fn rustls_server_config_builder_new() -> *mut rustls_server_config_builder {
         ffi_panic_boundary! {
             let builder = ServerConfigBuilder {
-                           base: rustls::ServerConfig::builder().with_safe_defaults(),
-                           verifier: NoClientAuth::boxed(),
-                           cert_resolver: None,
-                           session_storage: None,
-                           alpn_protocols: vec![],
-                           ignore_client_order: None,
-                       };
-                BoxCastPtr::to_mut_ptr(builder)
+                base: rustls::ServerConfig::builder(),
+                verifier: WebPkiClientVerifier::no_client_auth(),
+                cert_resolver: None,
+                session_storage: None,
+                alpn_protocols: vec![],
+                ignore_client_order: None,
+            };
+            to_boxed_mut_ptr(builder)
         }
     }
 
@@ -119,7 +110,8 @@ impl rustls_server_config_builder {
         builder_out: *mut *mut rustls_server_config_builder,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let cipher_suites: &[*const rustls_supported_ciphersuite] = try_slice!(cipher_suites, cipher_suites_len);
+            let cipher_suites: &[*const rustls_supported_ciphersuite] =
+                try_slice!(cipher_suites, cipher_suites_len);
             let mut cs_vec: Vec<SupportedCipherSuite> = Vec::new();
             for &cs in cipher_suites.iter() {
                 let cs = try_ref_from_ptr!(cs);
@@ -140,7 +132,12 @@ impl rustls_server_config_builder {
                 }
             }
 
-            let result = rustls::ServerConfig::builder().with_cipher_suites(&cs_vec).with_safe_default_kx_groups().with_protocol_versions(&versions);
+            let provider = rustls::crypto::CryptoProvider {
+                cipher_suites: cs_vec,
+                ..rustls::crypto::ring::default_provider()
+            };
+            let result = rustls::ServerConfig::builder_with_provider(provider.into())
+                .with_protocol_versions(&versions);
             let base = match result {
                 Ok(new) => new,
                 Err(_) => return rustls_result::InvalidParameter,
@@ -148,47 +145,28 @@ impl rustls_server_config_builder {
 
             let builder = ServerConfigBuilder {
                 base,
-                verifier: NoClientAuth::boxed(),
+                verifier: WebPkiClientVerifier::no_client_auth(),
                 cert_resolver: None,
                 session_storage: None,
                 alpn_protocols: vec![],
                 ignore_client_order: None,
             };
-            BoxCastPtr::set_mut_ptr(builder_out, builder);
+            set_boxed_mut_ptr(builder_out, builder);
             rustls_result::Ok
         }
     }
 
-    /// Create a rustls_server_config_builder for TLS sessions that require
-    /// valid client certificates. The passed rustls_client_cert_verifier may
-    /// be used in several builders.
-    /// For memory lifetime, see rustls_server_config_builder_new.
+    /// Create a rustls_server_config_builder for TLS sessions that may verify client
+    /// certificates. This increases the refcount of `verifier` and doesn't take ownership.
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_set_client_verifier(
         builder: *mut rustls_server_config_builder,
         verifier: *const rustls_client_cert_verifier,
-    ) {
-        ffi_panic_boundary! {
-        let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-        let verifier: Arc<AllowAnyAuthenticatedClient> = try_arc_from_ptr!(verifier);
-        builder.verifier = verifier;
-        }
-    }
-
-    /// Create a rustls_server_config_builder for TLS sessions that accept
-    /// valid client certificates, but do not require them. The passed
-    /// rustls_client_cert_verifier_optional may be used in several builders.
-    /// For memory lifetime, see rustls_server_config_builder_new.
-    #[no_mangle]
-    pub extern "C" fn rustls_server_config_builder_set_client_verifier_optional(
-        builder: *mut rustls_server_config_builder,
-        verifier: *const rustls_client_cert_verifier_optional,
     ) {
         ffi_panic_boundary! {
             let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-            let verifier: Arc<AllowAnyAnonymousOrAuthenticatedClient> = try_arc_from_ptr!(verifier);
-
-            builder.verifier = verifier;
+            let verifier = try_ref_from_ptr!(verifier);
+            builder.verifier = verifier.clone();
         }
     }
 
@@ -200,7 +178,7 @@ impl rustls_server_config_builder {
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_free(config: *mut rustls_server_config_builder) {
         ffi_panic_boundary! {
-            BoxCastPtr::to_box(config);
+            free_box(config);
         }
     }
 
@@ -269,13 +247,14 @@ impl rustls_server_config_builder {
         certified_keys_len: size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-        let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-        let keys_ptrs: &[*const rustls_certified_key] = try_slice!(certified_keys, certified_keys_len);
-        let mut keys: Vec<Arc<CertifiedKey>> = Vec::new();
-        for &key_ptr in keys_ptrs {
-            let certified_key: Arc<CertifiedKey> = try_arc_from_ptr!(key_ptr);
-            keys.push(certified_key);
-        }
+            let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
+            let keys_ptrs: &[*const rustls_certified_key] =
+                try_slice!(certified_keys, certified_keys_len);
+            let mut keys: Vec<Arc<CertifiedKey>> = Vec::new();
+            for &key_ptr in keys_ptrs {
+                let certified_key: Arc<CertifiedKey> = try_clone_arc!(key_ptr);
+                keys.push(certified_key);
+            }
             builder.cert_resolver = Some(Arc::new(ResolvesServerCertFromChoices::new(&keys)));
             rustls_result::Ok
         }
@@ -302,7 +281,7 @@ impl rustls_server_config_builder {
             if let Some(ignore_client_order) = builder.ignore_client_order {
                 config.ignore_client_order = ignore_client_order;
             }
-            ArcCastPtr::to_const_ptr(config)
+            to_arc_const_ptr(config)
         }
     }
 }
@@ -317,7 +296,7 @@ impl rustls_server_config {
     #[no_mangle]
     pub extern "C" fn rustls_server_config_free(config: *const rustls_server_config) {
         ffi_panic_boundary! {
-            rustls_server_config::free(config);
+            free_arc(config);
         }
     }
 
@@ -333,7 +312,7 @@ impl rustls_server_config {
         conn_out: *mut *mut rustls_connection,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let config: Arc<ServerConfig> = try_arc_from_ptr!(config);
+            let config: Arc<ServerConfig> = try_clone_arc!(config);
 
             let server_connection = match ServerConnection::new(config) {
                 Ok(sc) => sc,
@@ -343,7 +322,7 @@ impl rustls_server_config {
             // to the caller. After this point, we must return rustls_result::Ok so the
             // caller knows it is responsible for this memory.
             let c = Connection::from_server(server_connection);
-            BoxCastPtr::set_mut_ptr(conn_out, c);
+            set_boxed_mut_ptr(conn_out, c);
             rustls_result::Ok
         }
     }
@@ -366,10 +345,10 @@ pub extern "C" fn rustls_server_connection_get_server_name(
     ffi_panic_boundary! {
         let conn: &Connection = try_ref_from_ptr!(conn);
         if buf.is_null() {
-            return NullParameter
+            return NullParameter;
         }
         if out_n.is_null() {
-            return NullParameter
+            return NullParameter;
         }
         let server_connection = match conn.as_server() {
             Some(s) => s,
@@ -381,14 +360,12 @@ pub extern "C" fn rustls_server_connection_get_server_name(
                 unsafe {
                     *out_n = 0;
                 }
-                return rustls_result::Ok
-            },
+                return rustls_result::Ok;
+            }
         };
         let len: usize = sni_hostname.len();
         if len > count {
-            unsafe {
-                *out_n = 0
-            }
+            unsafe { *out_n = 0 }
             return rustls_result::InsufficientSize;
         }
         unsafe {
@@ -402,6 +379,7 @@ pub extern "C" fn rustls_server_connection_get_server_name(
 /// Choose the server certificate to be used for a connection based on certificate
 /// type. Will pick the first CertfiedKey available that is suitable for
 /// the SignatureSchemes supported by the client.
+#[derive(Debug)]
 struct ResolvesServerCertFromChoices {
     choices: Vec<Arc<CertifiedKey>>,
 }
@@ -452,7 +430,8 @@ pub struct rustls_client_hello<'a> {
     alpn: *const rustls_slice_slice_bytes<'a>,
 }
 
-impl<'a> CastPtr for rustls_client_hello<'a> {
+impl<'a> Castable for rustls_client_hello<'a> {
+    type Ownership = OwnershipRef;
     type RustType = rustls_client_hello<'a>;
 }
 
@@ -549,6 +528,12 @@ impl ResolvesServerCert for ClientHelloResolver {
 unsafe impl Sync for ClientHelloResolver {}
 unsafe impl Send for ClientHelloResolver {}
 
+impl Debug for ClientHelloResolver {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("ClientHelloResolver").finish()
+    }
+}
+
 impl rustls_server_config_builder {
     /// Register a callback to be invoked when a connection created from this config
     /// sees a TLS ClientHello message. If `userdata` has been set with
@@ -575,9 +560,7 @@ impl rustls_server_config_builder {
                 None => return rustls_result::NullParameter,
             };
             let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-            builder.cert_resolver = Some(Arc::new(ClientHelloResolver::new(
-                callback
-            )));
+            builder.cert_resolver = Some(Arc::new(ClientHelloResolver::new(callback)));
             rustls_result::Ok
         }
     }
@@ -585,27 +568,7 @@ impl rustls_server_config_builder {
 
 // Turn a slice of u16's into a vec of SignatureScheme as needed by rustls.
 fn sigschemes(input: &[u16]) -> Vec<SignatureScheme> {
-    use rustls::SignatureScheme::*;
-    input
-        .iter()
-        .map(|n| match n {
-            // TODO: Once rustls 0.20.0+ is released, we can use `.into()` instead of this match.
-            0x0201 => RSA_PKCS1_SHA1,
-            0x0203 => ECDSA_SHA1_Legacy,
-            0x0401 => RSA_PKCS1_SHA256,
-            0x0403 => ECDSA_NISTP256_SHA256,
-            0x0501 => RSA_PKCS1_SHA384,
-            0x0503 => ECDSA_NISTP384_SHA384,
-            0x0601 => RSA_PKCS1_SHA512,
-            0x0603 => ECDSA_NISTP521_SHA512,
-            0x0804 => RSA_PSS_SHA256,
-            0x0805 => RSA_PSS_SHA384,
-            0x0806 => RSA_PSS_SHA512,
-            0x0807 => ED25519,
-            0x0808 => ED448,
-            n => SignatureScheme::Unknown(*n),
-        })
-        .collect()
+    input.iter().copied().map(Into::into).collect()
 }
 
 /// Select a `rustls_certified_key` from the list that matches the cryptographic
@@ -631,11 +594,15 @@ pub extern "C" fn rustls_client_hello_select_certified_key(
 ) -> rustls_result {
     ffi_panic_boundary! {
         let hello = try_ref_from_ptr!(hello);
-        let schemes: Vec<SignatureScheme> = sigschemes(try_slice!(hello.signature_schemes.data, hello.signature_schemes.len));
+        let schemes: Vec<SignatureScheme> = sigschemes(try_slice!(
+            hello.signature_schemes.data,
+            hello.signature_schemes.len
+        ));
         if out_key.is_null() {
-            return NullParameter
+            return NullParameter;
         }
-        let keys_ptrs: &[*const rustls_certified_key] = try_slice!(certified_keys, certified_keys_len);
+        let keys_ptrs: &[*const rustls_certified_key] =
+            try_slice!(certified_keys, certified_keys_len);
         for &key_ptr in keys_ptrs {
             let key_ref: &CertifiedKey = try_ref_from_ptr!(key_ptr);
             if key_ref.key.choose_scheme(&schemes).is_some() {
@@ -673,9 +640,7 @@ impl rustls_server_config_builder {
                 None => return rustls_result::NullParameter,
             };
             let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-            builder.session_storage = Some(Arc::new(SessionStoreBroker::new(
-                get_cb, put_cb
-            )));
+            builder.session_storage = Some(Arc::new(SessionStoreBroker::new(get_cb, put_cb)));
             rustls_result::Ok
         }
     }
@@ -724,8 +689,8 @@ mod tests {
     fn test_server_connection_new() {
         let builder: *mut rustls_server_config_builder =
             rustls_server_config_builder::rustls_server_config_builder_new();
-        let cert_pem = include_str!("../localhost/cert.pem").as_bytes();
-        let key_pem = include_str!("../localhost/key.pem").as_bytes();
+        let cert_pem = include_str!("../testdata/localhost/cert.pem").as_bytes();
+        let key_pem = include_str!("../testdata/localhost/key.pem").as_bytes();
         let mut certified_key: *const rustls_certified_key = null();
         let result = rustls_certified_key::rustls_certified_key_build(
             cert_pem.as_ptr(),
