@@ -1,9 +1,8 @@
-use std::convert::TryFrom;
-
 use crate::error::rustls_result;
 use crate::rslice::rustls_slice_bytes;
 use crate::userdata_get;
 use libc::{c_int, c_void, size_t};
+use std::fmt::{Debug, Formatter};
 
 /// Any context information the callback will receive when invoked.
 pub type rustls_session_store_userdata = *mut c_void;
@@ -98,25 +97,24 @@ impl SessionStoreBroker {
         // See <https://github.com/rustls/rustls-ffi/pull/64#issuecomment-800766940>
         let mut data: Vec<u8> = vec![0; 65 * 1024];
         let mut out_n: size_t = 0;
-        unsafe {
-            let cb = self.get_cb;
-            let result = cb(
+
+        let cb = self.get_cb;
+        let result = unsafe {
+            cb(
                 userdata,
                 &key,
                 remove as c_int,
                 data.as_mut_ptr(),
                 data.len(),
                 &mut out_n,
-            );
-            let result: rustls_result =
-                rustls_result::try_from(result).unwrap_or(rustls_result::General);
-            match result {
-                rustls_result::Ok => {
-                    data.set_len(out_n);
-                    Some(data)
-                }
-                _ => None,
+            )
+        };
+        match rustls_result::from(result) {
+            rustls_result::Ok => {
+                unsafe { data.set_len(out_n) };
+                Some(data)
             }
+            _ => None,
         }
     }
 
@@ -151,6 +149,12 @@ impl rustls::server::StoresServerSessions for SessionStoreBroker {
     }
 }
 
+impl Debug for SessionStoreBroker {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("SessionStoreBroker").finish()
+    }
+}
+
 /// This struct can be considered thread safe, as long
 /// as the registered callbacks are thread safe. This is
 /// documented as a requirement in the API.
